@@ -45,6 +45,56 @@ from giscanner.transformer import Transformer
 from . import utils
 
 
+def process_wrapper_binary(option, opt_str, value, parser):
+    if value is None:
+        setattr(parser.values, option.dest, None)
+    else:
+        setattr(parser.values, option.dest, [value])
+
+
+def process_wrapper_args_begin(option, opt, value, parser, wrapper):
+    wrapper_cmd = getattr(parser.values, option.dest)
+    if wrapper_cmd is None:
+        parser.error(f"--{wrapper}-wrapper-args-begin must be called after --use-{wrapper}-wrapper")
+    while len(parser.rargs) > 0 and parser.rargs[0] != f"--{wrapper}-wrapper-args-end":
+        arg = parser.rargs.pop(0)
+        wrapper_cmd.append(arg)
+
+
+def process_wrapper_args_end(option, opt, value, parser):
+    pass
+
+
+def get_cross_compile_option_group(parser):
+    group = optparse.OptionGroup(parser, "Cross compilation options")
+
+    group.add_option("", "--use-binary-wrapper",
+                     dest="wrapper", type="string", default=None,
+                     help="wrapper to use for running programs (useful when cross-compiling)",
+                     action="callback", callback=process_wrapper_binary)
+    group.add_option("", "--binary-wrapper-args-begin",
+                     help="Start additional binary wrapper arguments",
+                     dest="wrapper", default=None,
+                     action="callback", callback=lambda option, opt, value, parser: process_wrapper_args_begin(option, opt, value, parser, "binary"))
+    group.add_option("", "--binary-wrapper-args-end",
+                     help="End additional binary wrapper arguments",
+                     action="callback", callback=process_wrapper_args_end)
+
+    group.add_option("", "--use-ldd-wrapper",
+                     dest="ldd_wrapper", type="string", default=None,
+                     help="wrapper to use instead of ldd (useful when cross-compiling)",
+                     action="callback", callback=process_wrapper_binary)
+    group.add_option("", "--ldd-wrapper-args-begin",
+                     help="Start additional ldd wrapper arguments",
+                     dest="ldd_wrapper", default=None,
+                     action="callback", callback=lambda option, opt, value, parser: process_wrapper_args_begin(option, opt, value, parser, "ldd"))
+    group.add_option("", "--ldd-wrapper-args-end",
+                     help="End additional ldd wrapper arguments",
+                     action="callback", callback=process_wrapper_args_end)
+
+    return group
+
+
 def process_cflags_begin(option, opt, value, parser):
     cflags = getattr(parser.values, option.dest)
     while len(parser.rargs) > 0 and parser.rargs[0] != '--cflags-end':
@@ -120,12 +170,10 @@ def _get_option_parser():
     parser.add_option("", "--program",
                       action="store", dest="program", default=None,
                       help="program to execute")
-    parser.add_option("", "--use-binary-wrapper",
-                      action="store", dest="wrapper", default=None,
-                      help="wrapper to use for running programs (useful when cross-compiling)")
-    parser.add_option("", "--use-ldd-wrapper",
-                      action="store", dest="ldd_wrapper", default=None,
-                      help="wrapper to use instead of ldd (useful when cross-compiling)")
+
+    group = get_cross_compile_option_group(parser)
+    parser.add_option_group(group)
+
     parser.add_option("", "--lib-dirs-envvar",
                       action="store", dest="lib_dirs_envvar", default=None,
                       help="environment variable to write a list of library directories to (for running the transient binary), instead of standard LD_LIBRARY_PATH")
@@ -220,6 +268,11 @@ match the namespace prefix.""")
     parser.add_option("", "--compiler",
                       action="store", dest="compiler", default=None,
                       help="the C compiler to use internally")
+    parser.add_option("", "--doc-format",
+                      action="store", dest="doc_format",
+                      help=("name of the documentation format used in the project, "
+                            "should be one of gi-docgen, gtk-doc-docbook, "
+                            "gtk-doc-markdown or hotdoc"))
 
     group = get_preprocessor_option_group(parser)
     parser.add_option_group(group)
@@ -296,7 +349,7 @@ def test_codegen(optstring,
                                       include_last_src)
         gen.write()
     else:
-        _error("Invaild namespace '%s'" % (namespace, ))
+        _error("Invalid namespace '%s'" % (namespace, ))
     return 0
 
 
@@ -433,16 +486,13 @@ def create_binary(transformer, options, args):
 
     shlibs = resolve_shlibs(options, binary, options.libraries)
     if options.wrapper:
-        # The wrapper needs the binary itself, not the libtool wrapper script,
-        # so we check if libtool has sneaked the binary into .libs subdirectory
-        # and adjust the path accordingly
-        import os.path
-        dir_name, binary_name = os.path.split(binary.args[0])
-        libtool_binary = os.path.join(dir_name, '.libs', binary_name)
-        if os.path.exists(libtool_binary):
-            binary.args[0] = libtool_binary
-        # Then prepend the wrapper to the command line to execute
-        binary.args = [options.wrapper] + binary.args
+        binary.args = options.wrapper + binary.args
+
+        libtool = utils.get_libtool_command(options)
+
+        if libtool is not None:
+            binary.args = libtool + ['--mode=execute'] + binary.args
+
     gdump_parser.set_introspection_binary(binary)
     gdump_parser.parse()
     return shlibs
@@ -473,6 +523,10 @@ def create_source_scanner(options, args):
     finally:
         for error in ss.get_errors():
             print(error, file=sys.stderr)
+
+    if ss.get_errors() and options.warn_fatal:
+        _error("error caught during scanner parsing")
+
     return ss, filenames
 
 
@@ -581,6 +635,15 @@ def scanner_main(args):
             or options.header_only):
         _error("Must specify --program or --library")
 
+    DOC_FORMATS = [
+        'gi-docgen',
+        'hotdoc',
+        'gtk-doc-markdown',
+        'gtk-doc-docbook',
+    ]
+    if options.doc_format and options.doc_format not in DOC_FORMATS:
+        _error("Unknown doc-format: %s" % (options.doc_format, ))
+
     namespace = create_namespace(options)
     logger = message.MessageLogger.get(namespace=namespace)
     if options.warn_all:
@@ -639,6 +702,8 @@ def scanner_main(args):
 
     transformer.namespace.c_includes = options.c_includes
     transformer.namespace.exported_packages = exported_packages
+    if options.doc_format:
+        transformer.namespace.doc_format = options.doc_format
 
     sources_top_dirs = get_source_root_dirs(options, filenames)
     writer = Writer(transformer.namespace, sources_top_dirs)
